//
//  PCI Mediaswitch driver, bits needed by the core kernel
//
//  Copyright (c) 2001, TiVo, Inc.  All rights reserved.
//

#include <linux/module.h>
#include <linux/pci.h>
#include <linux/init.h>
#include <linux/serial.h>
#include <linux/serial_reg.h>
#include <linux/delay.h>
#include <linux/irq.h>
#include <asm/px_hw.h>
#ifndef CPU_IRQ_INT0
#include <asm/bcm7020.h>
#endif
#include <asm/serial.h>
#include <asm/tivopromdefs.h>

#ifdef CONFIG_KDB
#include <linux/kdb.h>
#endif

#ifdef CONFIG_TIVO_DEVEL
extern int debug_irqs;
#define DEBUG_IRQS debug_irqs
#else
#define DEBUG_IRQS 0
#endif

extern struct irqaction cascade;
extern int tivo_setup_irq(int irq, struct irqaction *new);
extern asmlinkage void do_IRQ(int irq, struct pt_regs *regs);
extern void set_debug_traps(void);

#define PCI_VENDOR_ID_TIVO_TEST	0xeeee
#define PCI_VENDOR_ID_TIVO	0x1741
#define PCI_DEVICE_ID_MSWITCH	0x0000

struct fpga_data {
	struct pci_dev *dev;
	void *base;
	unsigned long base_bus;
	unsigned long size;
};

static struct fpga_data myData;

void fpga_gpio9_reset(void)
{
	cli();

	if (!myData.base) {
		return;
	}

	MsSetBits(MS_GPIOOB_REG,  0 << 9, 1 << 9);
	MsSetBits(MS_GPIOOBE_REG, 1 << 9, 1 << 9);

	mdelay(100);
}

static int __init fpga_probe(struct pci_dev *dev, const struct pci_device_id *id)
{
	pci_set_master(dev);
	if (pci_enable_device(dev)) {
		printk(KERN_ERR "FPGA: error enabling device");
		return -EIO;
	}
	myData.dev = dev;
	myData.base_bus = pci_resource_start(dev, 0);
	myData.size = pci_resource_len(dev, 0);
	myData.base = ioremap(myData.base_bus, myData.size);

	return 0;
}

static struct pci_device_id fpga_idtable[] __initdata = {
	{
		vendor:		PCI_VENDOR_ID_TIVO_TEST,
		device:		PCI_DEVICE_ID_MSWITCH,
		subvendor:	PCI_ANY_ID,
		subdevice:	PCI_ANY_ID,
	},
	{
		vendor:		PCI_VENDOR_ID_TIVO,
		device:		PCI_DEVICE_ID_MSWITCH,
		subvendor:	PCI_ANY_ID,
		subdevice:	PCI_ANY_ID,
	},
	{ 0, }
};

static struct pci_driver fpgacore_driver = {
	name:		"mediaswitch",
	id_table:	fpga_idtable,
	probe:		&fpga_probe,
};

// set port to offset because some parts of serial.c insist port !=0
#define TIVO_FPGA_SERIAL_INIT(offset,irq_nr) \
	{ baud_base: 33333333/16, port: offset, irq: irq_nr, \
	  flags: ASYNC_BOOT_AUTOCONF, io_type: SERIAL_IO_MEM, \
	  iomem_base: (void *)offset, iomem_reg_shift: 2 }

static struct serial_struct fpga_uart[] __initdata = {
	TIVO_FPGA_SERIAL_INIT(MS_UART1_BASE,IRQ_MS_UART1),
	TIVO_FPGA_SERIAL_INIT(MS_UART2_BASE,IRQ_MS_UART2),
	TIVO_FPGA_SERIAL_INIT(MS_UART3_BASE,IRQ_MS_UART3)
};

#define TIVO_LB_SERIAL_INIT(base, baud, irq_nr) \
	{ baud_base: baud, port: base, irq: irq_nr, \
	  flags: ASYNC_BOOT_AUTOCONF | ASYNC_FORCE, io_type: SERIAL_VADDR }

static struct serial_struct lb_uart[] __initdata = {
	TIVO_LB_SERIAL_INIT(TIVO_MODEM_BASE, BASE_BAUD, IRQ_MSG_GPIO13),
};

static int __initdata dsscon = 0;

static int __init dsscon_setup(char *str)
{
	if (strcmp(str, "true") == 0)
		dsscon = 1;
	else
		dsscon = 0;
	return 0;
}

__setup("dsscon=", dsscon_setup);

static int __initdata fpirblast = 0x0080;

static int __init fpirblast_setup(char *str)
{
	if (strcmp(str, "true") == 0)
		fpirblast = 0;
	return 1;
}

__setup("fpirblast=", fpirblast_setup);

int __init fpgacore_init(void)
{
	int result;
	int i;

	result = pci_module_init(&fpgacore_driver);
	if (result)
		return result;

	MsWrite(MS_CONTROL_REGISTER, 0x00012000);
	MsSetBits(MS_GPIOOB_REG, 0x0420 | fpirblast, ~0x001e);
	MsWrite(MS_GPIOOBE_REG, 0x04f7);
	MsWrite(MS_GPIOSEL0_REG, 0x00000000);
	MsWrite(MS_GPIOSEL1_REG, 0x00111000);
	MsWrite(MS_FAN_CTRL_REG, 0x010000ff);
	MsWrite(MS_IM_REG, 0);
	MsWrite(MS_HEIM_REG, 0);
	MsWrite(MS_TPIM_REG, 0);
	MsWrite(MS_GPIOIM_REG, 0);
	tivo_setup_irq(CPU_IRQ_INT0, &cascade);
	tivo_setup_irq(IRQ_MS_HE, &cascade);
	tivo_setup_irq(IRQ_MS_TP, &cascade);
	tivo_setup_irq(IRQ_MS_GPIO, &cascade);

#ifdef CONFIG_SERIAL
	for (i = 0; i < 3; i++) {
		fpga_uart[i].iomem_base =
		          (void *)( (unsigned long)fpga_uart[i].iomem_base +
	                            (unsigned long)myData.base );
	}
	register_serial(&fpga_uart[0]);
	register_serial(&lb_uart[0]);
	register_serial(&fpga_uart[2]);
	register_serial(&fpga_uart[1]);

#ifdef CONFIG_SERIAL_CONSOLE
	// need to make sure this doesn't happen until we're ready
	// for it, that's why it got yanked out of tty_io.c
	if (dsscon)
		serial_console_init();
#endif
#ifdef CONFIG_REMOTE_DEBUG
	set_debug_traps();
#endif
#ifdef CONFIG_KDB
	kdb_init();
	if (KDB_FLAG(EARLYKDB)) {
		KDB_ENTER();
	}
#endif
#endif
	return 0;
}

#if 0
// It would have been nice to just do it this way, but we need to use
// MsRead/MsWrite early (from IDE init), so instead call fpgacore_init
// from PCI init
module_init(fpgacore_init);
#endif

static inline void do_heirq(struct pt_regs *regs)
{
	unsigned int irqmask;
	int i;

	irqmask = MsRead(MS_HEIS_REG) & MsRead(MS_HEIM_REG);
	for (i = 15; i >= 0; i--)
		if (irqmask & (1 << i)) {
			do_IRQ(i + MEDIASWITCH_IHBASE, regs);
			return;
		}

	if (DEBUG_IRQS >= 4) {
		printk(KERN_WARNING "spurious HE int! MS_HEIS_REG=0x%x\n", MsRead(MS_HEIS_REG));
	}
	atomic_inc(&irq_err_count);
	return;
}

static inline void do_tpirq(struct pt_regs *regs)
{
	unsigned int irqmask;
	int i;

	irqmask = MsRead(MS_TPIS_REG) & MsRead(MS_TPIM_REG);
	for (i = 15; i >= 0; i--)
		if (irqmask & (1 << (i + 16))) {
			do_IRQ(i + MEDIASWITCH_ITBASE, regs);
			return;
		}

	if (DEBUG_IRQS >= 4) {
		printk(KERN_WARNING "spurious TP int! MS_TPIS_REG=0x%x\n", MsRead(MS_TPIS_REG));
	}
	atomic_inc(&irq_err_count);
	return;
}

static inline void do_gpioirq(struct pt_regs *regs)
{
	unsigned int irqmask;
	int i;

	irqmask = MsRead(MS_GPIOIS_REG) & MsRead(MS_GPIOIM_REG);
	for (i = 15; i >= 0; i--)
		if (irqmask & (1 << i)) {
			do_IRQ(i + MEDIASWITCH_IGBASE, regs);
			return;
		}

	if (DEBUG_IRQS >= 4) {
		printk(KERN_WARNING "spurious GPIO int! MS_GPIOIS_REG=0x%x\n", MsRead(MS_GPIOIS_REG));
	}
	atomic_inc(&irq_err_count);
	return;
}

void do_fpgairq(struct pt_regs *regs)
{
	unsigned int irqmask;
	int i;

	irqmask = MsRead(MS_ISR_REG) & MsRead(MS_IM_REG);
	for (i = 31; i >= 0; i--)
		if (irqmask & (1 << i)) {
			switch (i) {
			case 28:
				do_heirq(regs);
				return;
			case 29:
				do_tpirq(regs);
				return;
			case 30:
				do_gpioirq(regs);
				return;
			default:
				do_IRQ(i + MEDIASWITCH_IBASE, regs);
				return;
			}
		}

	if (DEBUG_IRQS >= 4) {
		printk(KERN_WARNING "spurious MS int! MS_ISR_REG=0x%x MS_IM_REG=0x%x\n", MsRead(MS_ISR_REG), MsRead(MS_IM_REG));
	}
	atomic_inc(&irq_err_count);
	return;
}

//
// The exported kernel interface.  Don't use these, use the macros in
// asm/px_hw.h without the leading underscore.
//

//
// _MsRead and _MsWrite read and write a Mediaswitch register at offset,
// caller doesn't have to worry about where the base is
//
unsigned int _MsRead(unsigned long offset)
{
	return readl(myData.base + offset);
}
EXPORT_SYMBOL(_MsRead);

void _MsWrite(unsigned long offset, unsigned int data)
{
	writel(data, myData.base + offset);
}
EXPORT_SYMBOL(_MsWrite);

void _MsSetBits(unsigned long offset, unsigned int data, unsigned int mask)
{
	unsigned long flags;

	save_flags(flags); cli();
	data = (data & mask) | (readl(myData.base + offset) & ~mask );
	writel(data, myData.base + offset);
	restore_flags(flags);
}
EXPORT_SYMBOL(_MsSetBits);

void _MsAndReg(unsigned long offset, unsigned int data)
{
	unsigned long flags;

	save_flags(flags); cli();
	data &= readl(myData.base + offset);
	writel(data, myData.base + offset);
	restore_flags(flags);
}
EXPORT_SYMBOL(_MsAndReg);

void _MsOrReg(unsigned long offset, unsigned int data)
{
	unsigned long flags;

	save_flags(flags); cli();
	data |= readl(myData.base + offset);
	writel(data, myData.base + offset);
	restore_flags(flags);
}
EXPORT_SYMBOL(_MsOrReg);

//
// _MsGetBase returns a base address pointer to the Mediaswitch registers,
// in case a driver needs to do its own reads and writes without the
// function call overhead
//
// This is an ioremapped value, so the return value _should_ be used with
// readl/writel.  At the very least, the user should be aware of the byte
// swapping behavior of the PCI bus.
//
// Returns 0 if no MS present
//
void *_MsGetBase(void)
{
	return myData.base;
}
EXPORT_SYMBOL(_MsGetBase);

//
// The bus address of the Mediaswitch registers
//
unsigned long _MsGetBusAddr(void)
{
	return myData.base_bus;
}
EXPORT_SYMBOL(_MsGetBusAddr);

//
// The size of the PCI memory region
//
unsigned long _MsGetSize(void)
{
	return myData.size;
}
EXPORT_SYMBOL(_MsGetSize);
