/*
 * Register defs for Broadcom BCM7315 (and similar) host controller
 *
 * Copyright (C) 2003-2004 TiVo, Inc. All Rights Reserved.
 */
#ifndef __ASM_BCM7315_H
#define __ASM_BCM7315_H

#include <asm/addrspace.h>

#define	BCM_LINUX_DMA_IRQ		1
#define	BCM_LINUX_EBI_RX_IRQ		2
#define	BCM_LINUX_EBI_TX_IRQ		3
#define	BCM_LINUX_IDE0_IRQ		4
#define	BCM_LINUX_IDE1_IRQ   		5
#define	BCM_LINUX_ADP_C_IRQ		6
#define	BCM_LINUX_ATP_C_IRQ		7
#define	BCM_LINUX_USB_HOST_IRQ		8
#define	BCM_LINUX_MODEM_IRQ		9
#define	BCM_LINUX_UPG_IRQ		10
#define	BCM_LINUX_SPI_IRQ		11
#define	BCM_LINUX_I2C_IRQ		12
#define	BCM_LINUX_MIPS_IRQ		15
#define	BCM_LINUX_POSTED_XFER_IRQ	16
#define	BCM_LINUX_BGE_IRQ		17
#define	BCM_LINUX_MINITITAN_IRQ		18
#define	BCM_LINUX_XPT_STATUS_IRQ	19
#define	BCM_LINUX_XPT_OVFLOW_IRQ	20
#define	BCM_LINUX_XPT_MSG_IRQ		21
#define	BCM_LINUX_XPT_ICAM_IRQ		22
#define	BCM_LINUX_ATP_A_IRQ		23
#define	BCM_LINUX_ADP_A_IRQ		24
#define	BCM_LINUX_ATP_AUD_IRQ		25
#define	BCM_LINUX_VEC_IRQ		27
#define	BCM_LINUX_UART_IRQ		28
#define	BCM_LINUX_SDS_IRQ		30
#define	BCM_LINUX_TIMR_IRQ		32
#define BCM_LINUX_UARTA_IRQ		33
#define BCM_LINUX_UARTB_IRQ		34
#define BCM_LINUX_SOFT_MODEM_DMA_IRQS	35

#define BCM_LINUX_UPG_KBD_IRQ		37
#define BCM_LINUX_UPG_RES1		38	
#define BCM_LINUX_UPG_IRB_IRQ		39
#define BCM_LINUX_UPG_UB_IRQ		40
#define BCM_LINUX_UPG_UA_IRQ		41
#define BCM_LINUX_UPG_RES2		42
#define BCM_LINUX_UPG_SCA_IRQ		43
#define BCM_LINUX_UPG_SCB_IRQ		44
// This one is for backwards compatibility:
#define BCM_LINUX_UPG_RES3		BCM_LINUX_UPG_SCB_IRQ
#define BCM_LINUX_UPG_IIC_IRQ		45
#define BCM_LINUX_UPG_SPI_IRQ		46
#define BCM_LINUX_UPG_GIO_IRQ		47
#define BCM_LINUX_UPG_ICAP_IRQ		48

#define BCM_LINUX_INTERNAL_IRQ		55
#define BCM_LINUX_EXTERNAL0_IRQ		56
#define BCM_LINUX_EXTERNAL1_IRQ		57
#define BCM_LINUX_EXTERNAL2_IRQ		58
#define BCM_LINUX_EXTERNAL3_IRQ		59
#define BCM_LINUX_SYSTIMER_IRQ		60
#define BCM_LINUX_EXTERNAL4_IRQ		61

// This is for GPIO 0 - 31 only
#define BCM_LINUX_GIO_IRQ_BASE		62
#define BCM_LINUX_GIO_IRQ(n)		(BCM_LINUX_GIO_IRQ_BASE + n)
#define BCM_LINUX_MGIO_IRQ_BASE		94
#define BCM_LINUX_MGIO_IRQ(n)		(BCM_LINUX_MGIO_IRQ_BASE + n)
// This is for GPIO 32 - 35
#define BCM_LINUX_GIOX_IRQ_BASE		126
#define BCM_LINUX_GIOX_IRQ(n)		(BCM_LINUX_GIOX_IRQ_BASE + n - 32)

// Note: if you increase this, also increase NR_IRQS in <asm/irq.h>
#define BCM_NR_IRQS			134


#ifndef _LANGUAGE_ASSEMBLY
#ifdef __KERNEL__
#include <linux/types.h>

#define __preg8		(volatile u8*)
#define __preg16	(volatile u16*)
#define __preg32	(volatile u32*)
#else /* !__KERNEL__ */
#define __preg8		(volatile unsigned char*)
#define __preg16	(volatile unsigned short*)
#define __preg32	(volatile unsigned int*)
#endif /* __KERNEL__ */
#else /* _LANGUAGE_ASSEMBLY */
#define __preg8
#define __preg16
#define __preg32
#endif /* !_LANGUAGE_ASSEMBLY */

#define BCM7315_REG_BASE 	0xfffe0000


#define __PERIPHREG(offset)	__preg8(BCM7315_REG_BASE + offset)

#define BCM_UARTA_BASE		__PERIPHREG(0xb0)
#define BCM_UARTB_BASE		__PERIPHREG(0xc0)

#define BCM_UART_RCVSTAT	0x00
#define BCM_UART_RCVDATA	0x01
#define BCM_UART_CONTROL	0x03
#define BCM_UART_BAUDHI		0x04
#define BCM_UART_BAUDLO		0x05
#define BCM_UART_XMTSTAT	0x06
#define BCM_UART_XMTDATA	0x07

#define BCM_PM1_IRQEN_HI	__PERIPHREG(0xfc + 0x00)
#define BCM_PM1_IRQEN_LO	__PERIPHREG(0xfc + 0x01)
#define BCM_PM1_IRQSTAT_HI	__PERIPHREG(0xfc + 0x02)
#define BCM_PM1_IRQSTAT_LO	__PERIPHREG(0xfc + 0x03)

#define BCM_GPIO_SEL0		__PERIPHREG(0x8007)
#define BCM_GIO_DIRHSB		__PERIPHREG(0x50 + 0x06)
#define BCM_GIO_DIRLSB		__PERIPHREG(0x50 + 0x07)
#define BCM_GIO_OUTHSB		__PERIPHREG(0x50 + 0x02)
#define BCM_GIO_OUTLSB		__PERIPHREG(0x50 + 0x03)
#define BCM_GIO_MASKMSB		__PERIPHREG(0x50 + 0x0f)
#define BCM_GIO_MASK2MSB	__PERIPHREG(0x50 + 0x1d)
#define BCM_GIO_MASKHSB		__PERIPHREG(0x50 + 0x10)
#define BCM_GIO_MASKLSB		__PERIPHREG(0x50 + 0x11)
#define BCM_GIO_RSTMSB		__PERIPHREG(0x50 + 0x12)
#define BCM_GIO_RST2MSB		__PERIPHREG(0x50 + 0x1e)
#define BCM_GIO_RSTHSB		__PERIPHREG(0x50 + 0x13)
#define BCM_GIO_RSTLSB		__PERIPHREG(0x50 + 0x14)
#define BCM_GIO_STATMSB		__PERIPHREG(0x50 + 0x15)
#define BCM_GIO_STAT2MSB	__PERIPHREG(0x50 + 0x1f)
#define BCM_GIO_STATHSB		__PERIPHREG(0x50 + 0x16)
#define BCM_GIO_STATLSB		__PERIPHREG(0x50 + 0x17)

#define BCM_MGIOSEL_BYTE3	__PERIPHREG(0xf0 + 0x03)
#define BCM_MGIO_DIRMSB		__PERIPHREG(0x70 + 0x08)
#define BCM_MGIO_ECMSB		__PERIPHREG(0x70 + 0x0c)
#define BCM_MGIO_EIMSB		__PERIPHREG(0x70 + 0x10)
#define BCM_MGIO_MASKMSB	__PERIPHREG(0x70 + 0x14)
#define BCM_MGIO_MASK2MSB	__PERIPHREG(0x70 + 0x15)
#define BCM_MGIO_MASKHSB	__PERIPHREG(0x70 + 0x16)
#define BCM_MGIO_MASKLSB	__PERIPHREG(0x70 + 0x17)
#define BCM_MGIO_RSTMSB		__PERIPHREG(0x70 + 0x18)
#define BCM_MGIO_RST2MSB	__PERIPHREG(0x70 + 0x19)
#define BCM_MGIO_RSTHSB		__PERIPHREG(0x70 + 0x1a)
#define BCM_MGIO_RSTLSB		__PERIPHREG(0x70 + 0x1b)
#define BCM_MGIO_STATMSB	__PERIPHREG(0x70 + 0x1c)
#define BCM_MGIO_STAT2MSB	__PERIPHREG(0x70 + 0x1d)
#define BCM_MGIO_STATHSB	__PERIPHREG(0x70 + 0x1e)
#define BCM_MGIO_STATLSB	__PERIPHREG(0x70 + 0x1f)

#define BCM_GIOEXT_OUTLSB	__PERIPHREG(0xa0 + 0x01)
#define BCM_GIOEXT_DIRLSB	__PERIPHREG(0xa0 + 0x03)
#define BCM_GIOEXT_ODLSB	__PERIPHREG(0xa0 + 0x05)
#define BCM_GIOEXT_ECLSB	__PERIPHREG(0xa0 + 0x07)
#define BCM_GIOEXT_EILSB	__PERIPHREG(0xa0 + 0x09)
#define BCM_GIOEXT_MASKLSB	__PERIPHREG(0xa0 + 0x0b)
#define BCM_GIOEXT_RSTLSB	__PERIPHREG(0xa0 + 0x0d)
#define BCM_GIOEXT_STATLSB	__PERIPHREG(0xa0 + 0x0f)

#define __INTCREG(offset)	__preg32(BCM7315_REG_BASE + 0x0600 + offset)

#define BCM_INTMASK		__INTCREG(0x0c)
#define BCM_INTSTATUS		__INTCREG(0x10)

#define __TIMRREG(offset)	__preg32(BCM7315_REG_BASE + 0x0700 + offset)

#define BCM_TINT		__TIMRREG(0x00)
#define BCM_TIMER0_CNTL		__TIMRREG(0x04)
#define BCM_TIMER1_CNTL		__TIMRREG(0x08)
#define BCM_TIMER2_CNTL		__TIMRREG(0x0c)
#define BCM_TIMER3_CNTL		__TIMRREG(0x10)
#define BCM_TIMER0_STAT		__TIMRREG(0x14)

#define __TMREG(offset)		__preg8(BCM7315_REG_BASE + 0x8000 + offset)

/* IDE MACROS */

#define BCM_IDE_BASE_7xxx	0xFFFB0000

#define __IDEREG8(offset)	__preg8(BCM_IDE_BASE_7xxx + offset)
#define __IDEREG16(offset)	__preg16(BCM_IDE_BASE_7xxx + offset)
#define __IDEREG32(offset)	__preg32(BCM_IDE_BASE_7xxx + offset)

#define BCM_IDE_CMD		__IDEREG32(0x004)
#define BCM_IDE_PROG_IF		__IDEREG32(0x008)
#define BCM_IDE_PRI_CS0_ADDR	__IDEREG32(0x010)
#define BCM_IDE_PRI_CS1_ADDR	__IDEREG32(0x014)
#define BCM_IDE_SEC_CS0_ADDR	__IDEREG32(0x018)
#define BCM_IDE_SEC_CS1_ADDR	__IDEREG32(0x01c)
#define BCM_IDE_BMIDE_ADDR	__IDEREG32(0x020)
#define BCM_IDE_PIO_TIMING	__IDEREG32(0x040)
#define BCM_IDE_DMA_TIMING	__IDEREG32(0x044)
#define BCM_IDE_PIO_CONTROL	__IDEREG32(0x048)
#define BCM_IDE_UDMA_CONTROL	__IDEREG32(0x054)
#define BCM_IDE_UDMA_MODE	__IDEREG32(0x058)
#define BCM_IDE_IF_CONTROL	__IDEREG32(0x064)
#define BCM_IDE_MBIST_CONTROL	__IDEREG32(0x068)
#define BCM_IDE_INT_MASK	__IDEREG32(0x080)
#define BCM_IDE_INT_STATUS	__IDEREG32(0x084)
#define BCM_IDE_PRI_DATA	__IDEREG16(0x200)
#define BCM_IDE_PRI_CMD_STAT	__IDEREG8(0x242)
#define BCM_IDE_PRI_BM_CMD_STAT	__IDEREG32(0x300)
#define BCM_IDE_PRI_BM_DESC_TAB	__IDEREG32(0x304)
#define BCM_IDE_SEC_BM_CMD_STAT	__IDEREG32(0x308)
#define BCM_IDE_SEC_BM_DESC_TAB	__IDEREG32(0x30c)
 

/**********************************************************************
 *IDE_CMD
 **********************************************************************/
/* IO_ACCESS [00:00] - boolean */
#define IDE_CMD_IO_ACCESS_ALIGN                         0
#define IDE_CMD_IO_ACCESS_BITS                          1
#define IDE_CMD_IO_ACCESS_MASK                          0x00000001UL
#define IDE_CMD_IO_ACCESS_SHIFT                         0
#define IDE_CMD_IO_ACCESS_ENABLE                        1
#define IDE_CMD_IO_ACCESS_DISABLE                       0

/* MASTER_ENA [02:02] - boolean */
#define IDE_CMD_MASTER_ENA_ALIGN                        0
#define IDE_CMD_MASTER_ENA_BITS                         1
#define IDE_CMD_MASTER_ENA_MASK                         0x00000004UL
#define IDE_CMD_MASTER_ENA_SHIFT                        2
#define IDE_CMD_MASTER_ENA_ENABLE                       1
#define IDE_CMD_MASTER_ENA_DISABLE                      0

/**********************************************************************
 *IDE_PROG_IF
 **********************************************************************/
/* PRI_NATIVE [08:08] - boolean */
#define IDE_PROG_IF_PRI_NATIVE_ALIGN                    0
#define IDE_PROG_IF_PRI_NATIVE_BITS                     1
#define IDE_PROG_IF_PRI_NATIVE_MASK                     0x00000100UL
#define IDE_PROG_IF_PRI_NATIVE_SHIFT                    8
#define IDE_PROG_IF_PRI_NATIVE_COMPAT_MODE              0
#define IDE_PROG_IF_PRI_NATIVE_NATIVE_MODE              1

/* PRI_DISABLE_REQ [09:09] - boolean */
#define IDE_PROG_IF_PRI_DISABLE_REQ_ALIGN               0
#define IDE_PROG_IF_PRI_DISABLE_REQ_BITS                1
#define IDE_PROG_IF_PRI_DISABLE_REQ_MASK                0x00000200UL
#define IDE_PROG_IF_PRI_DISABLE_REQ_SHIFT               9
#define IDE_PROG_IF_PRI_DISABLE_REQ_REQUEST             1

/* SEC_NATIVE [10:10] - boolean */
#define IDE_PROG_IF_SEC_NATIVE_ALIGN                    0
#define IDE_PROG_IF_SEC_NATIVE_BITS                     1
#define IDE_PROG_IF_SEC_NATIVE_MASK                     0x00000400UL
#define IDE_PROG_IF_SEC_NATIVE_SHIFT                    10
#define IDE_PROG_IF_SEC_NATIVE_COMPAT_MODE              0
#define IDE_PROG_IF_SEC_NATIVE_NATIVE_MODE              1

/* SEC_DISABLE_REQ [09:09] - boolean */
#define IDE_PROG_IF_SEC_DISABLE_REQ_ALIGN               0
#define IDE_PROG_IF_SEC_DISABLE_REQ_BITS                1
#define IDE_PROG_IF_SEC_DISABLE_REQ_MASK                0x00000800UL
#define IDE_PROG_IF_SEC_DISABLE_REQ_SHIFT               11
#define IDE_PROG_IF_SEC_DISABLE_REQ_REQUEST             1

/* MODE_SUPPORT [15:15] - boolean */
#define IDE_PROG_IF_MODE_SUPPORT_ALIGN                  0
#define IDE_PROG_IF_MODE_SUPPORT_BITS                   1
#define IDE_PROG_IF_MODE_SUPPORT_MASK                   0x00008000UL
#define IDE_PROG_IF_MODE_SUPPORT_SHIFT                  15
#define IDE_PROG_IF_MODE_SUPPORT_PIO_ONLY               0
#define IDE_PROG_IF_MODE_SUPPORT_DMA_SUPPORT            1

/**********************************************************************
 *IDE_PRI_CS0_ADD
 **********************************************************************/
/* P_CS0_BASE_ADDR [15:00] - unsigned */
#define IDE_PRI_CS0_ADD_P_CS0_BASE_ADDR_ALIGN           0
#define IDE_PRI_CS0_ADD_P_CS0_BASE_ADDR_BITS            16
#define IDE_PRI_CS0_ADD_P_CS0_BASE_ADDR_MASK            0x0000FFFFUL
#define IDE_PRI_CS0_ADD_P_CS0_BASE_ADDR_SHIFT           0

/**********************************************************************
 *IDE_PRI_CS1_ADD
 **********************************************************************/
/* P_CS1_BASE_ADDR [15:00] - unsigned */
#define IDE_PRI_CS1_ADD_P_CS1_BASE_ADDR_ALIGN           0
#define IDE_PRI_CS1_ADD_P_CS1_BASE_ADDR_BITS            16
#define IDE_PRI_CS1_ADD_P_CS1_BASE_ADDR_MASK            0x0000FFFFUL
#define IDE_PRI_CS1_ADD_P_CS1_BASE_ADDR_SHIFT           0

/**********************************************************************
 *IDE_SEC_CS0_ADD
 **********************************************************************/
/* S_CS0_BASE_ADDR [15:00] - unsigned */

/**********************************************************************
 *IDE_SEC_CS1_ADD
 **********************************************************************/
/* S_CS1_BASE_ADDR [15:00] - unsigned */

/**********************************************************************
 *IDE_BM_ADDR
 **********************************************************************/
/* IDE_BM_BASE_ADDR [15:00] - unsigned */
#define IDE_BM_ADDR_IDE_BM_BASE_ADDR_ALIGN              0
#define IDE_BM_ADDR_IDE_BM_BASE_ADDR_BITS               16
#define IDE_BM_ADDR_IDE_BM_BASE_ADDR_MASK               0x0000FFFFUL
#define IDE_BM_ADDR_IDE_BM_BASE_ADDR_SHIFT              0

/**********************************************************************
 *IDE_PIO_TIME
 **********************************************************************/
/* PS_PIO_CMD_REC [03:00] - unsigned */
#define IDE_PIO_TIME_PS_PIO_CMD_REC_ALIGN               0
#define IDE_PIO_TIME_PS_PIO_CMD_REC_BITS                4
#define IDE_PIO_TIME_PS_PIO_CMD_REC_MASK                0x0000000FUL
#define IDE_PIO_TIME_PS_PIO_CMD_REC_SHIFT               0

/* PS_PIO_CMD [07:04] - unsigned */
#define IDE_PIO_TIME_PS_PIO_CMD_ALIGN                   0
#define IDE_PIO_TIME_PS_PIO_CMD_BITS                    4
#define IDE_PIO_TIME_PS_PIO_CMD_MASK                    0x000000F0UL
#define IDE_PIO_TIME_PS_PIO_CMD_SHIFT                   4

/* PM_PIO_CMD_REC [11:08] - unsigned */
#define IDE_PIO_TIME_PM_PIO_CMD_REC_ALIGN               0
#define IDE_PIO_TIME_PM_PIO_CMD_REC_BITS                4
#define IDE_PIO_TIME_PM_PIO_CMD_REC_MASK                0x00000F00UL
#define IDE_PIO_TIME_PM_PIO_CMD_REC_SHIFT               8

/* PM_PIO_CMD [15:12] - unsigned */
#define IDE_PIO_TIME_PM_PIO_CMD_ALIGN                   0
#define IDE_PIO_TIME_PM_PIO_CMD_BITS                    4
#define IDE_PIO_TIME_PM_PIO_CMD_MASK                    0x0000F000UL
#define IDE_PIO_TIME_PM_PIO_CMD_SHIFT                   12


/* SS_PIO_CMD_REC [19:16] - unsigned */
#define IDE_PIO_TIME_SS_PIO_CMD_REC_ALIGN               0
#define IDE_PIO_TIME_SS_PIO_CMD_REC_BITS                4
#define IDE_PIO_TIME_SS_PIO_CMD_REC_MASK                0x000F0000UL
#define IDE_PIO_TIME_SS_PIO_CMD_REC_SHIFT               16

/* SS_PIO_CMD [23:20] - unsigned */
#define IDE_PIO_TIME_SS_PIO_CMD_ALIGN                   0
#define IDE_PIO_TIME_SS_PIO_CMD_BITS                    4
#define IDE_PIO_TIME_SS_PIO_CMD_MASK                    0x00F00000UL
#define IDE_PIO_TIME_SS_PIO_CMD_SHIFT                   20

/* SM_PIO_CMD_REC [27:24] - unsigned */
#define IDE_PIO_TIME_SM_PIO_CMD_REC_ALIGN               0
#define IDE_PIO_TIME_SM_PIO_CMD_REC_BITS                4
#define IDE_PIO_TIME_SM_PIO_CMD_REC_MASK                0x0F000000UL
#define IDE_PIO_TIME_SM_PIO_CMD_REC_SHIFT               24

/* SM_PIO_CMD [31:28] - unsigned */
#define IDE_PIO_TIME_SM_PIO_CMD_ALIGN                   0
#define IDE_PIO_TIME_SM_PIO_CMD_BITS                    4
#define IDE_PIO_TIME_SM_PIO_CMD_MASK                    0xF0000000UL
#define IDE_PIO_TIME_SM_PIO_CMD_SHIFT                   28

/**********************************************************************
 *IDE_DMA_TIME
 **********************************************************************/
/* PS_DMA_CMD_REC [03:00] - unsigned */
#define IDE_DMA_TIME_PS_DMA_CMD_REC_ALIGN               0
#define IDE_DMA_TIME_PS_DMA_CMD_REC_BITS                4
#define IDE_DMA_TIME_PS_DMA_CMD_REC_MASK                0x0000000FUL
#define IDE_DMA_TIME_PS_DMA_CMD_REC_SHIFT               0

/* PS_DMA_CMD [07:04] - unsigned */
#define IDE_DMA_TIME_PS_DMA_CMD_ALIGN                   0
#define IDE_DMA_TIME_PS_DMA_CMD_BITS                    4
#define IDE_DMA_TIME_PS_DMA_CMD_MASK                    0x000000F0UL
#define IDE_DMA_TIME_PS_DMA_CMD_SHIFT                   4

/* PM_DMA_CMD_REC [11:08] - unsigned */
#define IDE_DMA_TIME_PM_DMA_CMD_REC_ALIGN               0
#define IDE_DMA_TIME_PM_DMA_CMD_REC_BITS                4
#define IDE_DMA_TIME_PM_DMA_CMD_REC_MASK                0x00000F00UL
#define IDE_DMA_TIME_PM_DMA_CMD_REC_SHIFT               8

/* PM_DMA_CMD [15:12] - unsigned */
#define IDE_DMA_TIME_PM_DMA_CMD_ALIGN                   0
#define IDE_DMA_TIME_PM_DMA_CMD_BITS                    4
#define IDE_DMA_TIME_PM_DMA_CMD_MASK                    0x0000F000UL
#define IDE_DMA_TIME_PM_DMA_CMD_SHIFT                   12


/* SS_DMA_CMD_REC [19:16] - unsigned */
#define IDE_DMA_TIME_SS_DMA_CMD_REC_ALIGN               0
#define IDE_DMA_TIME_SS_DMA_CMD_REC_BITS                4
#define IDE_DMA_TIME_SS_DMA_CMD_REC_MASK                0x000F0000UL
#define IDE_DMA_TIME_SS_DMA_CMD_REC_SHIFT               16

/* SS_DMA_CMD [23:20] - unsigned */
#define IDE_DMA_TIME_SS_DMA_CMD_ALIGN                   0
#define IDE_DMA_TIME_SS_DMA_CMD_BITS                    4
#define IDE_DMA_TIME_SS_DMA_CMD_MASK                    0x00F00000UL
#define IDE_DMA_TIME_SS_DMA_CMD_SHIFT                   20

/* SM_DMA_CMD_REC [27:24] - unsigned */
#define IDE_DMA_TIME_SM_DMA_CMD_REC_ALIGN               0
#define IDE_DMA_TIME_SM_DMA_CMD_REC_BITS                4
#define IDE_DMA_TIME_SM_DMA_CMD_REC_MASK                0x0F000000UL
#define IDE_DMA_TIME_SM_DMA_CMD_REC_SHIFT               24

/* SM_DMA_CMD [31:28] - unsigned */
#define IDE_DMA_TIME_SM_DMA_CMD_ALIGN                   0
#define IDE_DMA_TIME_SM_DMA_CMD_BITS                    4
#define IDE_DMA_TIME_SM_DMA_CMD_MASK                    0xF0000000UL
#define IDE_DMA_TIME_SM_DMA_CMD_SHIFT                   28

/**********************************************************************
 *IDE_PIO_CNTL_MODE
 **********************************************************************/
/* DIS_PRI [00:00] - boolean */
#define IDE_PIO_CNTL_MODE_DIS_PRI_ALIGN                 0
#define IDE_PIO_CNTL_MODE_DIS_PRI_BITS                  1
#define IDE_PIO_CNTL_MODE_DIS_PRI_MASK                  0x00000001UL
#define IDE_PIO_CNTL_MODE_DIS_PRI_SHIFT                 0

/* PS_PST_WR_ENA [04:04] - boolean */
#define IDE_PIO_CNTL_MODE_PS_PST_WR_ENA_ALIGN           0
#define IDE_PIO_CNTL_MODE_PS_PST_WR_ENA_BITS            1
#define IDE_PIO_CNTL_MODE_PS_PST_WR_ENA_MASK            0x00000010UL
#define IDE_PIO_CNTL_MODE_PS_PST_WR_ENA_SHIFT           4

/* PM_PST_WR_ENA [05:05] - boolean */
#define IDE_PIO_CNTL_MODE_PM_PST_WR_ENA_ALIGN           0
#define IDE_PIO_CNTL_MODE_PM_PST_WR_ENA_BITS            1
#define IDE_PIO_CNTL_MODE_PM_PST_WR_ENA_MASK            0x00000020UL
#define IDE_PIO_CNTL_MODE_PM_PST_WR_ENA_SHIFT           5

/* PS_PRF_RD_ENA [06:06] - boolean */
#define IDE_PIO_CNTL_MODE_PS_PRF_RD_ENA_ALIGN           0
#define IDE_PIO_CNTL_MODE_PS_PRF_RD_ENA_BITS            1
#define IDE_PIO_CNTL_MODE_PS_PRF_RD_ENA_MASK            0x00000040UL
#define IDE_PIO_CNTL_MODE_PS_PRF_RD_ENA_SHIFT           6

/* PM_PRF_RD_ENA [07:07] - boolean */
#define IDE_PIO_CNTL_MODE_PM_PRF_RD_ENA_ALIGN           0
#define IDE_PIO_CNTL_MODE_PM_PRF_RD_ENA_BITS            1
#define IDE_PIO_CNTL_MODE_PM_PRF_RD_ENA_MASK            0x00000080UL
#define IDE_PIO_CNTL_MODE_PM_PRF_RD_ENA_SHIFT           7

/* DIS_SEC [08:08] - boolean */
#define IDE_PIO_CNTL_MODE_DIS_SEC_ALIGN                 0
#define IDE_PIO_CNTL_MODE_DIS_SEC_BITS                  1
#define IDE_PIO_CNTL_MODE_DIS_SEC_MASK                  0x00000100UL
#define IDE_PIO_CNTL_MODE_DIS_SEC_SHIFT                 8

/* SS_PST_WR_ENA [12:12] - boolean */
#define IDE_PIO_CNTL_MODE_SS_PST_WR_ENA_ALIGN           0
#define IDE_PIO_CNTL_MODE_SS_PST_WR_ENA_BITS            1
#define IDE_PIO_CNTL_MODE_SS_PST_WR_ENA_MASK            0x00001000UL
#define IDE_PIO_CNTL_MODE_SS_PST_WR_ENA_SHIFT           12

/* SM_PST_WR_ENA [13:13] - boolean */
#define IDE_PIO_CNTL_MODE_SM_PST_WR_ENA_ALIGN           0
#define IDE_PIO_CNTL_MODE_SM_PST_WR_ENA_BITS            1
#define IDE_PIO_CNTL_MODE_SM_PST_WR_ENA_MASK            0x00002000UL
#define IDE_PIO_CNTL_MODE_SM_PST_WR_ENA_SHIFT           13

/* SS_PRF_RD_ENA [14:14] - boolean */
#define IDE_PIO_CNTL_MODE_SS_PRF_RD_ENA_ALIGN           0
#define IDE_PIO_CNTL_MODE_SS_PRF_RD_ENA_BITS            1
#define IDE_PIO_CNTL_MODE_SS_PRF_RD_ENA_MASK            0x00004000UL
#define IDE_PIO_CNTL_MODE_SS_PRF_RD_ENA_SHIFT           14

/* SM_PRF_RD_ENA [15:15] - boolean */
#define IDE_PIO_CNTL_MODE_SM_PRF_RD_ENA_ALIGN           0
#define IDE_PIO_CNTL_MODE_SM_PRF_RD_ENA_BITS            1
#define IDE_PIO_CNTL_MODE_SM_PRF_RD_ENA_MASK            0x00008000UL
#define IDE_PIO_CNTL_MODE_SM_PRF_RD_ENA_SHIFT           15

/* PM_PIO_ACC_MD [18:16] - unsigned */
#define IDE_PIO_CNTL_MODE_PM_PIO_ACC_MD_ALIGN           0
#define IDE_PIO_CNTL_MODE_PM_PIO_ACC_MD_BITS            3
#define IDE_PIO_CNTL_MODE_PM_PIO_ACC_MD_MASK            0x00070000UL
#define IDE_PIO_CNTL_MODE_PM_PIO_ACC_MD_SHIFT           16

/* PS_PIO_ACC_MD [22:20] - unsigned */
#define IDE_PIO_CNTL_MODE_PS_PIO_ACC_MD_ALIGN           0
#define IDE_PIO_CNTL_MODE_PS_PIO_ACC_MD_BITS            3
#define IDE_PIO_CNTL_MODE_PS_PIO_ACC_MD_MASK            0x00700000UL
#define IDE_PIO_CNTL_MODE_PS_PIO_ACC_MD_SHIFT           20

/* SM_PIO_ACC_MD [26:24] - unsigned */
#define IDE_PIO_CNTL_MODE_SM_PIO_ACC_MD_ALIGN           0
#define IDE_PIO_CNTL_MODE_SM_PIO_ACC_MD_BITS            3
#define IDE_PIO_CNTL_MODE_SM_PIO_ACC_MD_MASK            0x07000000UL
#define IDE_PIO_CNTL_MODE_SM_PIO_ACC_MD_SHIFT           24

/* SS_PIO_ACC_MD [30:28] - unsigned */
#define IDE_PIO_CNTL_MODE_SS_PIO_ACC_MD_ALIGN           0
#define IDE_PIO_CNTL_MODE_SS_PIO_ACC_MD_BITS            3
#define IDE_PIO_CNTL_MODE_SS_PIO_ACC_MD_MASK            0x70000000UL
#define IDE_PIO_CNTL_MODE_SS_PIO_ACC_MD_SHIFT           28

/**********************************************************************
 *IDE_UDMA_CNTL_STAT
 **********************************************************************/
/* PM_UDMA [00:00] - boolean */
#define IDE_UDMA_CNTL_STAT_PM_UDMA_ALIGN                0
#define IDE_UDMA_CNTL_STAT_PM_UDMA_BITS                 1
#define IDE_UDMA_CNTL_STAT_PM_UDMA_MASK                 0x00000001UL
#define IDE_UDMA_CNTL_STAT_PM_UDMA_SHIFT                0
#define IDE_UDMA_CNTL_STAT_PM_UDMA_DISABLE              0
#define IDE_UDMA_CNTL_STAT_PM_UDMA_ENABLE               1

/* PS_UDMA [01:01] - boolean */
#define IDE_UDMA_CNTL_STAT_PS_UDMA_ALIGN                0
#define IDE_UDMA_CNTL_STAT_PS_UDMA_BITS                 1
#define IDE_UDMA_CNTL_STAT_PS_UDMA_MASK                 0x00000002UL
#define IDE_UDMA_CNTL_STAT_PS_UDMA_SHIFT                1
#define IDE_UDMA_CNTL_STAT_PS_UDMA_DISABLE              0
#define IDE_UDMA_CNTL_STAT_PS_UDMA_ENABLE               1

/* SM_UDMA [02:02] - boolean */
#define IDE_UDMA_CNTL_STAT_SM_UDMA_ALIGN                0
#define IDE_UDMA_CNTL_STAT_SM_UDMA_BITS                 1
#define IDE_UDMA_CNTL_STAT_SM_UDMA_MASK                 0x00000004UL
#define IDE_UDMA_CNTL_STAT_SM_UDMA_SHIFT                2
#define IDE_UDMA_CNTL_STAT_SM_UDMA_DISABLE              0
#define IDE_UDMA_CNTL_STAT_SM_UDMA_ENABLE               1

/* SS_UDMA [03:03] - boolean */
#define IDE_UDMA_CNTL_STAT_SS_UDMA_ALIGN                0
#define IDE_UDMA_CNTL_STAT_SS_UDMA_BITS                 1
#define IDE_UDMA_CNTL_STAT_SS_UDMA_MASK                 0x00000008UL
#define IDE_UDMA_CNTL_STAT_SS_UDMA_SHIFT                3
#define IDE_UDMA_CNTL_STAT_SS_UDMA_DISABLE              0
#define IDE_UDMA_CNTL_STAT_SS_UDMA_ENABLE               1

/* REPRT_MD [07:07] - boolean */
#define IDE_UDMA_CNTL_STAT_REPRT_MD_ALIGN               0
#define IDE_UDMA_CNTL_STAT_REPRT_MD_BITS                1
#define IDE_UDMA_CNTL_STAT_REPRT_MD_MASK                0x00000080UL
#define IDE_UDMA_CNTL_STAT_REPRT_MD_SHIFT               7
						 
/* EXTRA_DATA [08:08] - boolean */
#define IDE_UDMA_CNTL_STAT_PRI_EXTRA_ALIGN              0
#define IDE_UDMA_CNTL_STAT_PRI_EXTRA_BITS               1
#define IDE_UDMA_CNTL_STAT_PRI_EXTRA_MASK               0x00000100UL
#define IDE_UDMA_CNTL_STAT_PRI_EXTRA_SHIFT              8

/* EXTRA_DATA [12:12] - boolean */
#define IDE_UDMA_CNTL_STAT_SEC_EXTRA_ALIGN              0
#define IDE_UDMA_CNTL_STAT_SEC_EXTRA_BITS               1
#define IDE_UDMA_CNTL_STAT_SEC_EXTRA_MASK               0x00001000UL
#define IDE_UDMA_CNTL_STAT_SEC_EXTRA_SHIFT              12

/* PM_UDMA_MODE [18:16] - unsigned */
#define IDE_UDMA_CNTL_STAT_PM_UDMA_MODE_ALIGN           0
#define IDE_UDMA_CNTL_STAT_PM_UDMA_MODE_BITS            3
#define IDE_UDMA_CNTL_STAT_PM_UDMA_MODE_MASK            0x00070000UL
#define IDE_UDMA_CNTL_STAT_PM_UDMA_MODE_SHIFT           16
						  
/* PS_UDMA_MODE [22:20] - unsigned */
#define IDE_UDMA_CNTL_STAT_PS_UDMA_MODE_ALIGN           0
#define IDE_UDMA_CNTL_STAT_PS_UDMA_MODE_BITS            3
#define IDE_UDMA_CNTL_STAT_PS_UDMA_MODE_MASK            0x00700000UL
#define IDE_UDMA_CNTL_STAT_PS_UDMA_MODE_SHIFT           20
						 
/* SM_UDMA_MODE [26:24] - unsigned */
#define IDE_UDMA_CNTL_STAT_SM_UDMA_MODE_ALIGN           0
#define IDE_UDMA_CNTL_STAT_SM_UDMA_MODE_BITS            3
#define IDE_UDMA_CNTL_STAT_SM_UDMA_MODE_MASK            0x07000000UL
#define IDE_UDMA_CNTL_STAT_SM_UDMA_MODE_SHIFT           24

/* SS_UDMA_MODE [30:28] - unsigned */
#define IDE_UDMA_CNTL_STAT_SS_UDMA_MODE_ALIGN           0
#define IDE_UDMA_CNTL_STAT_SS_UDMA_MODE_BITS            3
#define IDE_UDMA_CNTL_STAT_SS_UDMA_MODE_MASK            0x70000000UL
#define IDE_UDMA_CNTL_STAT_SS_UDMA_MODE_SHIFT           28

/**********************************************************************
 *IDE_UDMA_MODE
 **********************************************************************/
/* MAX_PIO_BURST [07:00] - unsigned */
#define IDE_UDMA_MODE_MAX_PIO_BURST_ALIGN               0
#define IDE_UDMA_MODE_MAX_PIO_BURST_BITS                8
#define IDE_UDMA_MODE_MAX_PIO_BURST_MASK                0x000000FFUL
#define IDE_UDMA_MODE_MAX_PIO_BURST_SHIFT               0
						 
/* MIN_DMA_BURST [08:16] - unsigned */
#define IDE_UDMA_MODE_MIN_DMA_BURST_ALIGN               0
#define IDE_UDMA_MODE_MIN_DMA_BURST_BITS                8
#define IDE_UDMA_MODE_MIN_DMA_BURST_MASK                0x0000FF00UL
#define IDE_UDMA_MODE_MIN_DMA_BURST_SHIFT               8

/**********************************************************************
 *IDE_IF_CNTL_STAT
 **********************************************************************/
/* LB_PST_WR_ENA [16:16] - boolean */
#define IDE_IF_CNTL_STAT_LB_PST_WR_ENA_ALIGN            0
#define IDE_IF_CNTL_STAT_LB_PST_WR_ENA_BITS             1
#define IDE_IF_CNTL_STAT_LB_PST_WR_ENA_MASK             0x00010000UL
#define IDE_IF_CNTL_STAT_LB_PST_WR_ENA_SHIFT            16

/* DT_BYTE_SWP_ENA [17:17] - boolean */
#define IDE_IF_CNTL_STAT_DT_BYTE_SWP_ENA_ALIGN          0
#define IDE_IF_CNTL_STAT_DT_BYTE_SWP_ENA_BITS           1
#define IDE_IF_CNTL_STAT_DT_BYTE_SWP_ENA_MASK           0x00020000UL
#define IDE_IF_CNTL_STAT_DT_BYTE_SWP_ENA_SHIFT          17

/* DATA_SWP_TYPE [19:18] - unsigned */
#define IDE_IF_CNTL_STAT_DATA_SWP_TYPE_ALIGN            0
#define IDE_IF_CNTL_STAT_DATA_SWP_TYPE_BITS             2
#define IDE_IF_CNTL_STAT_DATA_SWP_TYPE_MASK             0x000C0000UL
#define IDE_IF_CNTL_STAT_DATA_SWP_TYPE_SHIFT            18

/* PIO_WR_WAIT [24:24] - boolean */
#define IDE_IF_CNTL_STAT_PIO_WR_WAIT_ALIGN              0
#define IDE_IF_CNTL_STAT_PIO_WR_WAIT_BITS               1
#define IDE_IF_CNTL_STAT_PIO_WR_WAIT_MASK               0x01000000UL
#define IDE_IF_CNTL_STAT_PIO_WR_WAIT_SHIFT              24

/* PIO_RD_WAIT [25:25] - boolean */
#define IDE_IF_CNTL_STAT_PIO_RD_WAIT_ALIGN              0
#define IDE_IF_CNTL_STAT_PIO_RD_WAIT_BITS               1
#define IDE_IF_CNTL_STAT_PIO_RD_WAIT_MASK               0x02000000UL
#define IDE_IF_CNTL_STAT_PIO_RD_WAIT_SHIFT              25


/**********************************************************************
 *IDE_MBIST_CNTL_STAT
 **********************************************************************/
/* MBIST_ENA_RD_FIFO [00:00] - boolean */
#define IDE_MBIST_CNTL_STAT_MBIST_ENA_RD_FIFO_ALIGN     0
#define IDE_MBIST_CNTL_STAT_MBIST_ENA_RD_FIFO_BITS      1
#define IDE_MBIST_CNTL_STAT_MBIST_ENA_RD_FIFO_MASK      0x00000001UL
#define IDE_MBIST_CNTL_STAT_MBIST_ENA_RD_FIFO_SHIFT     0

/* MBIST_HOLD_RD_FIFO [01:01] - boolean */
#define IDE_MBIST_CNTL_STAT_MBIST_HOLD_RD_FIFO_ALIGN    0
#define IDE_MBIST_CNTL_STAT_MBIST_HOLD_RD_FIFO_BITS     1
#define IDE_MBIST_CNTL_STAT_MBIST_HOLD_RD_FIFO_MASK     0x00000002UL
#define IDE_MBIST_CNTL_STAT_MBIST_HOLD_RD_FIFO_SHIFT    1

/* MBIST_DBG_ENA_RD_FIFO [02:02] - boolean */
#define IDE_MBIST_CNTL_STAT_MBIST_DBG_ENA_RD_FIFO_ALIGN 0
#define IDE_MBIST_CNTL_STAT_MBIST_DBG_ENA_RD_FIFO_BITS  1
#define IDE_MBIST_CNTL_STAT_MBIST_DBG_ENA_RD_FIFO_MASK  0x00000004UL
#define IDE_MBIST_CNTL_STAT_MBIST_DBG_ENA_RD_FIFO_SHIFT 2

/* MBIST_DMP_RD_FIFO [03:03] - boolean */
#define IDE_MBIST_CNTL_STAT_MBIST_DMP_RD_FIFO_ALIGN     0
#define IDE_MBIST_CNTL_STAT_MBIST_DMP_RD_FIFO_BITS      1
#define IDE_MBIST_CNTL_STAT_MBIST_DMP_RD_FIFO_MASK      0x00000008UL
#define IDE_MBIST_CNTL_STAT_MBIST_DMP_RD_FIFO_SHIFT     3

/* MBIST_ENA_WR_FIFO [04:04] - boolean */
#define IDE_MBIST_CNTL_STAT_MBIST_ENA_WR_FIFO_ALIGN     0
#define IDE_MBIST_CNTL_STAT_MBIST_ENA_WR_FIFO_BITS      1
#define IDE_MBIST_CNTL_STAT_MBIST_ENA_WR_FIFO_MASK      0x00000010UL
#define IDE_MBIST_CNTL_STAT_MBIST_ENA_WR_FIFO_SHIFT     4

/* MBIST_HOLD_WR_FIFO [05:05] - boolean */
#define IDE_MBIST_CNTL_STAT_MBIST_HOLD_WR_FIFO_ALIGN    0
#define IDE_MBIST_CNTL_STAT_MBIST_HOLD_WR_FIFO_BITS     1
#define IDE_MBIST_CNTL_STAT_MBIST_HOLD_WR_FIFO_MASK     0x00000020UL
#define IDE_MBIST_CNTL_STAT_MBIST_HOLD_WR_FIFO_SHIFT    5

/* MBIST_DBG_ENA_WR_FIFO [06:06] - boolean */
#define IDE_MBIST_CNTL_STAT_MBIST_DBG_ENA_WR_FIFO_ALIGN 0
#define IDE_MBIST_CNTL_STAT_MBIST_DBG_ENA_WR_FIFO_BITS  1
#define IDE_MBIST_CNTL_STAT_MBIST_DBG_ENA_WR_FIFO_MASK  0x00000040UL
#define IDE_MBIST_CNTL_STAT_MBIST_DBG_ENA_WR_FIFO_SHIFT 6

/* MBIST_DMP_WR_FIFO [07:07] - boolean */
#define IDE_MBIST_CNTL_STAT_MBIST_DMP_WR_FIFO_ALIGN     0
#define IDE_MBIST_CNTL_STAT_MBIST_DMP_WR_FIFO_BITS      1
#define IDE_MBIST_CNTL_STAT_MBIST_DMP_WR_FIFO_MASK      0x00000080UL
#define IDE_MBIST_CNTL_STAT_MBIST_DMP_WR_FIFO_SHIFT     7

/* MBIST_DONE_RD_FIFO [08:08] - boolean */
#define IDE_MBIST_CNTL_STAT_MBIST_DONE_RD_FIFO_ALIGN    0
#define IDE_MBIST_CNTL_STAT_MBIST_DONE_RD_FIFO_BITS     1
#define IDE_MBIST_CNTL_STAT_MBIST_DONE_RD_FIFO_MASK     0x00000100UL
#define IDE_MBIST_CNTL_STAT_MBIST_DONE_RD_FIFO_SHIFT    8

/* MBIST_DBG_OUT_RD_FIFO [09:09] - boolean */
#define IDE_MBIST_CNTL_STAT_MBIST_DBG_OUT_RD_FIFO_ALIGN 0
#define IDE_MBIST_CNTL_STAT_MBIST_DBG_OUT_RD_FIFO_BITS  1
#define IDE_MBIST_CNTL_STAT_MBIST_DBG_OUT_RD_FIFO_MASK  0x00000200UL
#define IDE_MBIST_CNTL_STAT_MBIST_DBG_OUT_RD_FIFO_SHIFT 9

/* MBIST_FAIL_RD_FIFO [10:10] - boolean */
#define IDE_MBIST_CNTL_STAT_MBIST_FAIL_RD_FIFO_ALIGN    0
#define IDE_MBIST_CNTL_STAT_MBIST_FAIL_RD_FIFO_BITS     1
#define IDE_MBIST_CNTL_STAT_MBIST_FAIL_RD_FIFO_MASK     0x00000400UL
#define IDE_MBIST_CNTL_STAT_MBIST_FAIL_RD_FIFO_SHIFT    10

/* MBIST_DONE_WR_FIFO [12:12] - boolean */
#define IDE_MBIST_CNTL_STAT_MBIST_DONE_WR_FIFO_ALIGN    0
#define IDE_MBIST_CNTL_STAT_MBIST_DONE_WR_FIFO_BITS     1
#define IDE_MBIST_CNTL_STAT_MBIST_DONE_WR_FIFO_MASK     0x00001000UL
#define IDE_MBIST_CNTL_STAT_MBIST_DONE_WR_FIFO_SHIFT    12

/* MBIST_DBG_OUT_WR_FIFO [13:13] - boolean */
#define IDE_MBIST_CNTL_STAT_MBIST_DBG_OUT_WR_FIFO_ALIGN 0
#define IDE_MBIST_CNTL_STAT_MBIST_DBG_OUT_WR_FIFO_BITS  1
#define IDE_MBIST_CNTL_STAT_MBIST_DBG_OUT_WR_FIFO_MASK  0x00002000UL
#define IDE_MBIST_CNTL_STAT_MBIST_DBG_OUT_WR_FIFO_SHIFT 13

/* MBIST_FAIL_WR_FIFO [14:14] - boolean */
#define IDE_MBIST_CNTL_STAT_MBIST_FAIL_WR_FIFO_ALIGN    0
#define IDE_MBIST_CNTL_STAT_MBIST_FAIL_WR_FIFO_BITS     1
#define IDE_MBIST_CNTL_STAT_MBIST_FAIL_WR_FIFO_MASK     0x00004000UL
#define IDE_MBIST_CNTL_STAT_MBIST_FAIL_WR_FIFO_SHIFT    14


/**********************************************************************
 *IDE_PRI_DTE_A
 **********************************************************************/
/* DT_START_ADDR [31:00] - unsigned */
#define IDE_PRI_DTE_A_DT_START_ADDR_ALIGN               0
#define IDE_PRI_DTE_A_DT_START_ADDR_BITS                32
#define IDE_PRI_DTE_A_DT_START_ADDR_MASK                0xFFFFFFFFUL
#define IDE_PRI_DTE_A_DT_START_ADDR_SHIFT               0


/**********************************************************************
 *IDE_PRI_DTE_B
 **********************************************************************/
/* DT_BYTE_CNT [15:00] - unsigned */
#define IDE_PRI_DTE_B_DT_BYTE_CNT_ALIGN                 0
#define IDE_PRI_DTE_B_DT_BYTE_CNT_BITS                  16
#define IDE_PRI_DTE_B_DT_BYTE_CNT_MASK                  0x0000FFFFUL
#define IDE_PRI_DTE_B_DT_BYTE_CNT_SHIFT                 0

/* EOT [31:31] - boolean */
#define IDE_PRI_DTE_B_EOT_ALIGN                         0
#define IDE_PRI_DTE_B_EOT_BITS                          1
#define IDE_PRI_DTE_B_EOT_MASK                          0x80000000UL
#define IDE_PRI_DTE_B_EOT_SHIFT                         31


/**********************************************************************
 *IDE_PRI_INT_MSK
 **********************************************************************/
/* IDE_MASK [00:00] - boolean */
#define IDE_PRI_INT_MSK_IDE_MASK_ALIGN                  0
#define IDE_PRI_INT_MSK_IDE_MASK_BITS                   1
#define IDE_PRI_INT_MSK_IDE_MASK_MASK                   0x00000001UL
#define IDE_PRI_INT_MSK_IDE_MASK_SHIFT                  0

/* EOT_MASK [01:01] - boolean */
#define IDE_PRI_INT_MSK_EOT_MASK_ALIGN                  0
#define IDE_PRI_INT_MSK_EOT_MASK_BITS                   1
#define IDE_PRI_INT_MSK_EOT_MASK_MASK                   0x00000002UL
#define IDE_PRI_INT_MSK_EOT_MASK_SHIFT                  1


/**********************************************************************
 *IDE_PRI_INT
 **********************************************************************/
/* EOT_INT [01:01] - boolean */
#define IDE_PRI_INT_EOT_INT_ALIGN                       0
#define IDE_PRI_INT_EOT_INT_BITS                        1
#define IDE_PRI_INT_EOT_INT_MASK                        0x00000002UL
#define IDE_PRI_INT_EOT_INT_SHIFT                       1

				   
/**********************************************************************
 *IDE_PRI_DATA
 **********************************************************************/
/* PIO_DATA [15:00] - unsigned */
#define IDE_PRI_DATA_PIO_DATA_ALIGN                     0
#define IDE_PRI_DATA_PIO_DATA_BITS                      16
#define IDE_PRI_DATA_PIO_DATA_MASK                      0x0000FFFFUL
#define IDE_PRI_DATA_PIO_DATA_SHIFT                     0
				   

/**********************************************************************
 *IDE_PRI_FEAT_ERR
 **********************************************************************/
/* FEAT_ERROR [07:00] - unsigned */
#define IDE_PRI_FEAT_ERR_FEAT_ERROR_ALIGN               0
#define IDE_PRI_FEAT_ERR_FEAT_ERROR_BITS                8
#define IDE_PRI_FEAT_ERR_FEAT_ERROR_MASK                0x000000FFUL
#define IDE_PRI_FEAT_ERR_FEAT_ERROR_SHIFT               0
				   

/**********************************************************************
 *IDE_PRI_SEC_CNT
 **********************************************************************/
/* SECTOR_CNT [07:00] - unsigned */
#define IDE_PRI_SEC_CNT_SECTOR_CNT_ALIGN                0
#define IDE_PRI_SEC_CNT_SECTOR_CNT_BITS                 8
#define IDE_PRI_SEC_CNT_SECTOR_CNT_MASK                 0x000000FFUL
#define IDE_PRI_SEC_CNT_SECTOR_CNT_SHIFT                0


/**********************************************************************
 *IDE_PRI_SEC_NUM
 **********************************************************************/
/* SECTOR_NUM [07:00] - unsigned */
#define IDE_PRI_SEC_NUM_SECTOR_NUM_ALIGN                0
#define IDE_PRI_SEC_NUM_SECTOR_NUM_BITS                 8
#define IDE_PRI_SEC_NUM_SECTOR_NUM_MASK                 0x000000FFUL
#define IDE_PRI_SEC_NUM_SECTOR_NUM_SHIFT                0


/**********************************************************************
 *IDE_PRI_CYL_LOW
 **********************************************************************/
/* CYL_LOW [07:00] - unsigned */
#define IDE_PRI_CYL_LOW_CYL_LOW_ALIGN                   0
#define IDE_PRI_CYL_LOW_CYL_LOW_BITS                    8
#define IDE_PRI_CYL_LOW_CYL_LOW_MASK                    0x000000FFUL
#define IDE_PRI_CYL_LOW_CYL_LOW_SHIFT                   0


/**********************************************************************
 *IDE_PRI_CYL_HI
 **********************************************************************/
/* CYL_HI [07:00] - unsigned */
#define IDE_PRI_CYL_HI_CYL_HI_ALIGN                     0
#define IDE_PRI_CYL_HI_CYL_HI_BITS                      8
#define IDE_PRI_CYL_HI_CYL_HI_MASK                      0x000000FFUL
#define IDE_PRI_CYL_HI_CYL_HI_SHIFT                     0


/**********************************************************************
 *IDE_PRI_DEV_HD
 **********************************************************************/
/* DEV_HD [07:00] - unsigned */
#define IDE_PRI_DEV_HD_DEV_HD_ALIGN                     0
#define IDE_PRI_DEV_HD_DEV_HD_BITS                      8
#define IDE_PRI_DEV_HD_DEV_HD_MASK                      0x000000FFUL
#define IDE_PRI_DEV_HD_DEV_HD_SHIFT                     0


/**********************************************************************
 *IDE_PRI_CMD_STAT
 **********************************************************************/
/* CMD_STAT [07:00] - unsigned */
#define IDE_PRI_CMD_STAT_CMD_STAT_ALIGN                 0
#define IDE_PRI_CMD_STAT_CMD_STAT_BITS                  8
#define IDE_PRI_CMD_STAT_CMD_STAT_MASK                  0x000000FFUL
#define IDE_PRI_CMD_STAT_CMD_STAT_SHIFT                 0
					
					  
/**********************************************************************
 *IDE_PRI_DEV_C_STAT
 **********************************************************************/
/* DEV_CNTL_ALT_STAT [07:00] - unsigned */
#define IDE_PRI_DEV_C_STAT_DEV_CNTL_ALT_STAT_ALIGN      0
#define IDE_PRI_DEV_C_STAT_DEV_CNTL_ALT_STAT_BITS       8
#define IDE_PRI_DEV_C_STAT_DEV_CNTL_ALT_STAT_MASK       0x000000FFUL
#define IDE_PRI_DEV_C_STAT_DEV_CNTL_ALT_STAT_SHIFT      0


/**********************************************************************
 *IDE_PRI_BM_CMD_STAT
 **********************************************************************/
/* START_STOP [00:00] - boolean */
#define IDE_PRI_BM_CMD_STAT_START_STOP_ALIGN            0
#define IDE_PRI_BM_CMD_STAT_START_STOP_BITS             1
#define IDE_PRI_BM_CMD_STAT_START_STOP_MASK             0x00000001UL
#define IDE_PRI_BM_CMD_STAT_START_STOP_SHIFT            0

/* RD_WR [03:03] - boolean */
#define IDE_PRI_BM_CMD_STAT_RD_WR_ALIGN                 0
#define IDE_PRI_BM_CMD_STAT_RD_WR_BITS                  1
#define IDE_PRI_BM_CMD_STAT_RD_WR_MASK                  0x00000008UL
#define IDE_PRI_BM_CMD_STAT_RD_WR_SHIFT                 3

/* ACTIVE [16:16] - boolean */
#define IDE_PRI_BM_CMD_STAT_ACTIVE_ALIGN                0
#define IDE_PRI_BM_CMD_STAT_ACTIVE_BITS                 1
#define IDE_PRI_BM_CMD_STAT_ACTIVE_MASK                 0x00010000UL
#define IDE_PRI_BM_CMD_STAT_ACTIVE_SHIFT                16

/* INTRQ [18:18] - boolean */
#define IDE_PRI_BM_CMD_STAT_INTRQ_ALIGN                 0
#define IDE_PRI_BM_CMD_STAT_INTRQ_BITS                  1
#define IDE_PRI_BM_CMD_STAT_INTRQ_MASK                  0x00040000UL
#define IDE_PRI_BM_CMD_STAT_INTRQ_SHIFT                 18

/* DRV0_DMA [21:21] - boolean */
#define IDE_PRI_BM_CMD_STAT_DRV0_DMA_ALIGN              0
#define IDE_PRI_BM_CMD_STAT_DRV0_DMA_BITS               1
#define IDE_PRI_BM_CMD_STAT_DRV0_DMA_MASK               0x00200000UL
#define IDE_PRI_BM_CMD_STAT_DRV0_DMA_SHIFT              21     

/* DRV1_DMA [22:22] - boolean */
#define IDE_PRI_BM_CMD_STAT_DRV1_DMA_ALIGN              0
#define IDE_PRI_BM_CMD_STAT_DRV1_DMA_BITS               1
#define IDE_PRI_BM_CMD_STAT_DRV1_DMA_MASK               0x00400000UL
#define IDE_PRI_BM_CMD_STAT_DRV1_DMA_SHIFT              22

/* SIMPLEX [23:23] - boolean */
#define IDE_PRI_BM_CMD_STAT_SIMPLEX_ALIGN               0
#define IDE_PRI_BM_CMD_STAT_SIMPLEX_BITS                1
#define IDE_PRI_BM_CMD_STAT_SIMPLEX_MASK                0x00800000UL
#define IDE_PRI_BM_CMD_STAT_SIMPLEX_SHIFT               23


/**********************************************************************
 *IDE_PRI_DT_PT
 **********************************************************************/
/* DT_PTR [31:00] - unsigned */
#define IDE_PRI_DT_PT_DT_PTR_ALIGN                      0
#define IDE_PRI_DT_PT_DT_PTR_BITS                       32
#define IDE_PRI_DT_PT_DT_PTR_MASK                       0xFFFFFFFFUL
#define IDE_PRI_DT_PT_DT_PTR_SHIFT                      0

/* END IDE MACROS */

/**********************************************************************
 *ISB/EBI DMA registers
 **********************************************************************/

#define __DMAREG(offset)	__preg32(BCM7315_REG_BASE + offset)

#define BCM_DMA_STATUS                                  __DMAREG(0x6000)
#define BCM_DMA_BYTE_REVERSE                            __DMAREG(0x6004)
#define BCM_DMA_HALF_REVERSE                            __DMAREG(0x6008)

#define BCM_DMA_CH1_CONFIG                              __DMAREG(0x6040)
#define BCM_DMA_CH1_MAX_BURST                           __DMAREG(0x6044)
#define BCM_DMA_CH1_BUFF_ADDR                           __DMAREG(0x6048)
#define BCM_DMA_CH1_BCNTR_LEN                           __DMAREG(0x604C)
#define BCM_DMA_CH1_BUFF_STAT                           __DMAREG(0x6050)
#define BCM_DMA_CH1_INT_STAT                            __DMAREG(0x6054)
#define BCM_DMA_CH1_INT_MASK                            __DMAREG(0x6058)
#define BCM_DMA_CH1_RING_OFFSET                         __DMAREG(0x6064)

#define BCM_DMA_CH2_CONFIG                              __DMAREG(0x6080)
#define BCM_DMA_CH2_MAX_BURST                           __DMAREG(0x6084)
#define BCM_DMA_CH2_BUFF_ADDR                           __DMAREG(0x6088)
#define BCM_DMA_CH2_BCNTR_LEN                           __DMAREG(0x608C)
#define BCM_DMA_CH2_BUFF_STAT                           __DMAREG(0x6090)
#define BCM_DMA_CH2_INT_STAT                            __DMAREG(0x6094)
#define BCM_DMA_CH2_INT_MASK                            __DMAREG(0x6098)
#define BCM_DMA_CH2_RING_OFFSET                         __DMAREG(0x60A4)

#define BCM_DMA_CH3_CONFIG                              __DMAREG(0x60C0)
#define BCM_DMA_CH3_MAX_BURST                           __DMAREG(0x60C4)
#define BCM_DMA_CH3_BUFF_ADDR                           __DMAREG(0x60C8)
#define BCM_DMA_CH3_BCNTR_LEN                           __DMAREG(0x60CC)
#define BCM_DMA_CH3_BUFF_STAT                           __DMAREG(0x60D0)
#define BCM_DMA_CH3_INT_STAT                            __DMAREG(0x60D4)
#define BCM_DMA_CH3_INT_MASK                            __DMAREG(0x60D8)
#define BCM_DMA_CH3_RING_OFFSET                         __DMAREG(0x60E4)

#define BCM_DMA_CH4_CONFIG                              __DMAREG(0x6100)
#define BCM_DMA_CH4_MAX_BURST                           __DMAREG(0x6104)
#define BCM_DMA_CH4_BUFF_ADDR                           __DMAREG(0x6108)
#define BCM_DMA_CH4_BCNTR_LEN                           __DMAREG(0x610C)
#define BCM_DMA_CH4_BUFF_STAT                           __DMAREG(0x6110)
#define BCM_DMA_CH4_INT_STAT                            __DMAREG(0x6114)
#define BCM_DMA_CH4_INT_MASK                            __DMAREG(0x6118)
#define BCM_DMA_CH4_RING_OFFSET                         __DMAREG(0x6124)

#define BCM_EBI_CS0_BASE				__DMAREG(0x7000) 
#define BCM_EBI_CS0_CTRL				__DMAREG(0x7004) 
#define BCM_EBI_CS1_BASE				__DMAREG(0x7008) 
#define BCM_EBI_CS1_CTRL				__DMAREG(0x700C)
#define BCM_EBI_CS2_BASE				__DMAREG(0x7010) 
#define BCM_EBI_CS2_CTRL				__DMAREG(0x7014) 
#define BCM_EBI_CS3_BASE				__DMAREG(0x7018) 
#define BCM_EBI_CS3_CTRL				__DMAREG(0x701C) 

#define BCM_EBI_CONFIG					__DMAREG(0x7100) 
#define BCM_EBI_MBIST					__DMAREG(0x7108) 
#define BCM_EBI_MBIST_STATUS				__DMAREG(0x710C) 
#define BCM_EBI_DMA_CTRL				__DMAREG(0x7200) 
#define BCM_EBI_DMA_RX_START_ADDR_1			__DMAREG(0x7204) 
#define BCM_EBI_DMA_RX_BUFFER_SIZE_1			__DMAREG(0x7208) 
#define BCM_EBI_DMA_TX_START_ADDR_1			__DMAREG(0x720C) 
#define BCM_EBI_DMA_TX_BUFFER_SIZE_1			__DMAREG(0x7210) 
#define BCM_EBI_DMA_RX_START_ADDR_2			__DMAREG(0x7214) 
#define BCM_EBI_DMA_RX_BUFFER_SIZE_2			__DMAREG(0x7218) 
#define BCM_EBI_DMA_TX_START_ADDR_2			__DMAREG(0x721C) 
#define BCM_EBI_DMA_TX_BUFFER_SIZE_2			__DMAREG(0x7220) 
#define BCM_EBI_DMA_STATUS				__DMAREG(0x7224) 

#define BCM_CHIPID                                      __preg32(0xFFFE0600)

#endif /* __ASM_BCM7315_H */
