/*
 * fasttcp.h
 * Header file for faster USER/TCP/DEVICE interface.
 *
 * Copyright TiVo (C) 2005
 *
 */
#ifndef _LINUX_FAST_TCP_H
#define _LINUX_FAST_TCP_H

#include <linux/types.h>
#include <linux/tcp.h>
#include <asm/ioctl.h>


#define FAST_TCP_MAX_MSS 1592  /* Reasonable for ethernet, but... ? */
#define FAST_TCP_MAX_HEADER 256 /* Damn this should be configurable from k->u */

#define FAST_TCP_FLAG_USERCALC_CRC_PARTIAL  (1<<0)
#define FAST_TCP_FLAG_MORE_PACKETS          (1<<1)
#define FAST_TCP_FLAG_PAGE_LOCKED           (1<<29)
#define FAST_TCP_FLAG_TRANSMIT_FINISHED     (1<<30)
#define FAST_TCP_FLAG_HEADER_CALCULATED     (1<<31)

struct fast_tcp_pkt {
	__u32 flags;
	__u32 crc_partial;
	__u32 mss;
	__u32 size;
#ifdef __KERNEL__
	union {
		wait_queue_head_t txfin;
#endif  /* __KERNEL__ */
		__u8 stuff_0[8];
#ifdef __KERNEL__
	};
#endif  /* __KERNEL__ */

#ifdef __KERNEL__
	
	union {
		struct sk_buff skb;
#endif  /* __KERNEL__ */
		__u8 stuff_1[176];
#ifdef __KERNEL__
	};
#endif  /* __KERNEL__ */
	__u8 pktdata[FAST_TCP_MAX_MSS + FAST_TCP_MAX_HEADER] ;
} __attribute__((packed));

#ifdef __KERNEL__
/* Actual IOCTL implementations in code */
ssize_t do_tcp_send_fast_skbs(struct sock *sk, unsigned int uaddr);
int do_tcp_wait_fast_skb(struct sock *sk, unsigned int uaddr);
int do_tcp_release_fast_skb(struct sock *sk, unsigned int uaddr);

#endif  /* __KERNEL__ */

#define FAST_TCP_TRANSMIT_PKTS     _IOW(0xc4, 1, struct fast_tcp_pkt)
#define FAST_TCP_RELEASE_PKTS      _IOW(0xc4, 2, struct fast_tcp_pkt)
#define FAST_TCP_TRANSMIT_WAIT_FIN _IOW(0xc4, 3, struct fast_tcp_pkt)

#endif	/* _LINUX_FAST_TCP_H */
